package org.richfaces.testng;

import org.ajax4jsf.template.Template;
import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class DropDownMenuTest extends SeleniumTestBase implements RichSeleniumTest {

    public DropDownMenuTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeMethod
    @Parameters({"browser", "filterPrefix"})
    public void startSelenium(String browser, String filterPrefix) {
    	super.startSelenium(browser, filterPrefix);
    }
    
    
    @BeforeMethod
    @Parameters({"loadStyleStrategy", "loadScriptStrategy"})
    protected void loadConfiguration(String loadStyleStrategy, String loadScriptStrategy) throws Exception {
	super.loadConfiguration(loadStyleStrategy, loadScriptStrategy);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterMethod(alwaysRun=true)
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testDropDownMenuComponent() throws Exception {
        _testDropDownMenuComponent(Template.SIMPLE);
        _testDropDownMenuComponent(Template.DATA_TABLE);
        _testDropDownMenuComponent(Template.MODAL_PANEL);
    }

    private void _testDropDownMenuComponent(Template template) {
        renderPage(template);

        String parentId = getParentId() + "_form:";
        String file = parentId + "file";
        String open = parentId + "open:anchor";
        String saveAs = parentId + "saveAs:anchor";
        String save = parentId + "save:anchor";
        String saveAll = parentId + "saveAll:anchor";
        String close = parentId + "close:anchor";
        String exit = parentId + "exit:anchor";
        String operation = parentId + "operation";

        writeStatus("Check menu item in ajax mode");

        selenium.mouseOver(file);
        selenium.mouseOver(saveAs);
        clickById(saveAll);

        AssertTextEquals(operation, "Save All");

        selenium.mouseOver(file);
        selenium.mouseOver(saveAs);
        clickById(save);

        AssertTextEquals(operation, "Save");

        writeStatus("Check the drop down menu closed");

        AssertNotVisible(save);
        AssertNotVisible(saveAs);

        writeStatus("Check menu item in server mode");

        selenium.mouseOver(file);
        clickCommandAndWait(close);

        AssertTextEquals(operation, "Close");

        writeStatus("Check menu item in 'none' mode");

        selenium.mouseOver(file);
        selenium.mouseOver(exit);
        clickById(exit);

        AssertTextEquals(operation, "Close");
        AssertNotVisible(exit);

    }

    public String getTestUrl() {
        return "pages/dropDownMenu/dropDownMenuTest.xhtml";
    }

}
