package org.richfaces.testng;

import org.ajax4jsf.template.Template;
import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class KeepAliveTest extends SeleniumTestBase implements RichSeleniumTest {

    public KeepAliveTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeMethod
    @Parameters({"browser", "filterPrefix"})
    public void startSelenium(String browser, String filterPrefix) {
    	super.startSelenium(browser, filterPrefix);
    }
    
    
    @BeforeMethod
    @Parameters({"loadStyleStrategy", "loadScriptStrategy"})
    protected void loadConfiguration(String loadStyleStrategy, String loadScriptStrategy) throws Exception {
	super.loadConfiguration(loadStyleStrategy, loadScriptStrategy);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterMethod(alwaysRun=true)
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testKeepAliveComponent() throws Exception {
    	_testKeepAliveComponent(Template.SIMPLE);
        _testKeepAliveComponent(Template.DATA_TABLE);
        _testKeepAliveComponent(Template.MODAL_PANEL);
    }

    private void _testKeepAliveComponent(Template template) {
        renderPage(template);

        String parentId = getParentId();

        String sumElemId = parentId + "sum";
        AssertValueEquals(sumElemId, "0");
        String addButtonId = parentId + "form:add";
        String addAjaxButtonId = parentId + "form:addAjax";

        writeStatus("The sum has to be increased by 5 twice");

        clickCommandAndWait(addButtonId);
        clickCommandAndWait(addButtonId);
        AssertValueEquals(sumElemId, "10");

        writeStatus("The sum has to be increased by 5 twice again");

        clickById(addAjaxButtonId);
        clickById(addAjaxButtonId);
        waitForAjaxCompletion();
        AssertValueEquals(sumElemId, "20");

        writeStatus("Test ajax only regime ... ");

        String sumAjaxOnlyElemId = parentId + "sumAjaxOnly";
        AssertValueEquals(sumAjaxOnlyElemId, "0");
        addButtonId = parentId + "formAjaxOnly:add";
        addAjaxButtonId = parentId + "formAjaxOnly:addAjax";

        writeStatus("State is not saved between full submits. The sum always is 5");

        clickCommandAndWait(addButtonId);
        clickCommandAndWait(addButtonId);
        AssertValueNotEquals(sumAjaxOnlyElemId, "10");

        writeStatus("Ajax request! The sum has to be increased by 5 twice");

//      This test doesn't pass! Bug?!

//      clickById(addAjaxButtonId);
//      clickById(addAjaxButtonId);
//      waitForAjaxCompletion();
//      AssertValueEquals(sumAjaxOnlyElemId, "10");
    }

    public String getTestUrl() {
        return "pages/keepAlive/keepAliveTest.xhtml";
    }

}
