/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.testng;

import org.ajax4jsf.template.Template;
import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class SimpleTogglePanelTest extends SeleniumTestBase implements RichSeleniumTest {

    public SimpleTogglePanelTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeMethod
    @Parameters({"browser", "filterPrefix"})
    public void startSelenium(String browser, String filterPrefix) {
    	super.startSelenium(browser, filterPrefix);
    }
    
    
    @BeforeMethod
    @Parameters({"loadStyleStrategy", "loadScriptStrategy"})
    protected void loadConfiguration(String loadStyleStrategy, String loadScriptStrategy) throws Exception {
	super.loadConfiguration(loadStyleStrategy, loadScriptStrategy);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterMethod(alwaysRun=true)
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testSimpleTogglePanelComponent() throws Exception {
        _testSimpleTogglePanelComponent(Template.SIMPLE);
        _testSimpleTogglePanelComponent(Template.DATA_TABLE);
        _testSimpleTogglePanelComponent(Template.MODAL_PANEL);
    }

    private void _testSimpleTogglePanelComponent(Template template) {
        renderPage(template);
        String parentId = getParentId() + "_form:";
        String inputId = parentId + "_value";
        String outputId = parentId + "_value2";

        String ajaxHeader = parentId + "panel1_header";
        String serverHeader = parentId + "panel2_header";
        String clientHeader = parentId + "panel3_header";

        String ajaxBody = parentId + "panel1_body";
        String serverBody = parentId + "panel2_body";
        String clientBody = parentId + "panel3_body";

        writeStatus("Click on client simple toggle panel. It should be opened.");
        clickById(clientHeader);
        Assert.assertTrue(isVisibleById(clientBody));
        writeStatus("Click on client simple toggle panel again. It should be closed.");
        clickById(clientHeader);
        Assert.assertFalse(isVisibleById(clientBody));

        writeStatus("Click on ajax simple toggle panel. It should be opened.");
        clickById(ajaxHeader);
        waitForAjaxCompletion();
        AssertValueEquals(inputId, "panel1");
        AssertTextEquals(outputId, "1");
        Assert.assertTrue(isVisibleById(ajaxBody));

        writeStatus("Click on ajax simple toggle panel again. It should be closed.");
        clickById(ajaxHeader);
        waitForAjaxCompletion();
        AssertValueEquals(inputId, "panel1");
        AssertTextEquals(outputId, "2");
        Assert.assertFalse(isVisibleById(ajaxBody));

        writeStatus("Click on server simple toggle panel. It should be opened.");
        clickCommandAndWait(serverHeader);
        AssertValueEquals(inputId, "panel2");
        AssertTextEquals(outputId, "3");
        Assert.assertTrue(isVisibleById(serverBody));

        writeStatus("Click on server simple toggle panel. It should be closed.");
        clickCommandAndWait(serverHeader);
        AssertValueEquals(inputId, "panel2");
        AssertTextEquals(outputId, "4");
        Assert.assertFalse(isVisibleById(serverBody));
    }

    public String getTestUrl() {
        return "pages/simpleTogglePanel/simpleTogglePanel.xhtml";
    }

}
