/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.testng;

import org.ajax4jsf.template.Template;
import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class SpacerTest extends SeleniumTestBase implements RichSeleniumTest {

    public SpacerTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeMethod
    @Parameters({"browser", "filterPrefix"})
    public void startSelenium(String browser, String filterPrefix) {
    	super.startSelenium(browser, filterPrefix);
    }
    
    
    @BeforeMethod
    @Parameters({"loadStyleStrategy", "loadScriptStrategy"})
    protected void loadConfiguration(String loadStyleStrategy, String loadScriptStrategy) throws Exception {
	super.loadConfiguration(loadStyleStrategy, loadScriptStrategy);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterMethod(alwaysRun=true)
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testSimpleTogglePanelComponent() throws Exception {
        _testSimpleTogglePanelComponent(Template.SIMPLE);
        _testSimpleTogglePanelComponent(Template.DATA_TABLE);
        _testSimpleTogglePanelComponent(Template.MODAL_PANEL);
    }

    private void _testSimpleTogglePanelComponent(Template template) {
        renderPage(template);
        String parentId = getParentId();
        String spacerId = parentId + "sp";

        writeStatus("Check width ");
        assertWidth(1 + 2, spacerId);

        writeStatus("Check height ");
        assertHeight(5 + 2, spacerId);

        writeStatus("Check styleClass ");
        assertStyleClass("sp", spacerId);

        writeStatus("Check style ");
        assertStyleClass("border: solid 1px green;", spacerId);
    }

    private void assertWidth(int width, String spacerId) {
        StringBuffer script = new StringBuffer(" document.getElementById('");
        script.append(spacerId);
        script.append("').offsetWidth");

        String w = runScript(script.toString());
        Assert.assertEquals(w, String.valueOf(width));
    }

    private void assertHeight(int height, String spacerId) {
        StringBuffer script = new StringBuffer(" document.getElementById('");
        script.append(spacerId);
        script.append("').offsetHeight");

        String h = runScript(script.toString());
        Assert.assertEquals(h, String.valueOf(height));
    }

    private void assertStyleClass(String styleClass, String spacerId) {
        StringBuffer script = new StringBuffer(" document.getElementById('");
        script.append(spacerId);
        script.append("').className");

        String sc = runScript(script.toString());
        if (sc != null) {
            Assert.assertTrue(sc.endsWith("sp"));
        } else {
            Assert.fail("ClassName is null");
        }
    }

    private void assertStyle(String style, String spacerId) {
        StringBuffer script = new StringBuffer(" document.getElementById('");
        script.append(spacerId);
        script.append("').style");

        String s = runScript(script.toString());
        Assert.assertEquals(s, style);
    }

    public String getTestUrl() {
        return "pages/spacer/spacerTest.xhtml";
    }

}
